`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    15:35:27 05/11/2016 
// Design Name: 
// Module Name:    lab10_2 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module lab10_2(
   clk, // clock from crystal oscillator
	rst_n, // sctive low reset
	col_n, // pressed column index
	row_n, // scanned row index
	display, // 14-segment display
	ftsd_ctl, // 14-segment display scan control
	pressed, // whether keypad pressed (1) or not (0)
	audio_appsel,
	audio_sysclk,
	audio_bck,
	audio_ws,
	audio_data
    );
	 
	input clk, rst_n;
	input [3:0] col_n;
	output [3:0] row_n;
	output [14:0] display;
	output [3:0] ftsd_ctl;
	output pressed;
	output audio_appsel;
	output audio_sysclk;
	output audio_bck;
	output audio_ws;
	output audio_data;
	
	wire clk_d;
	wire clk_debounce;

	wire [3:0] key;
	wire [1:0] ftsd_ctl_en;
	wire [3:0] ftsd_in;
	
	wire [3:0] dis3, dis2, dis1, dis0;
	wire [19:0] note;
	

	wire [15:0] audio_in_left;
	wire [15:0] audio_in_right;
		
	// Frequency divider
	freq_div U_1(
	  .clk(clk), // clock from the 40MHz oscillator
	  .rst_n(rst_n), // low active reset
	  .clk_out(clk_d), //divided clock output
	  .clk_debounce(clk_debounce), // clock control for debounce circuit
	  .clk_ctl(ftsd_ctl_en) // divided clock for 14-segment display scan
	);

	// Keypad scan
	keypad_scan U_2(
	  .clk(clk_debounce), // scan clock
	  .rst_n(rst_n), // active low reset
	  .col_n(col_n), // pressed column index
	  .row_n(row_n), // scanned row index
	  .key(key), // returned pressed key
	  .pressed(pressed) // whether key pressed (1) or not (0)
	);

	//Decide the note
	note_deal U_3(
		.key(key),
		.note(note),
		.dis3(dis3),
		.dis2(dis2),
		.dis1(dis1),
		.dis0(dis0),
		.pressed(pressed)
	);

	//Deal with note
	buzzer_ctl U_4(
			.clk(clk),
			.rst_n(rst_n),
			.note_div(note),
			.audio_left(audio_in_left),
			.audio_right(audio_in_right)
		 );
	speaker_ctl U_5(
			.audio_in_left(audio_in_left),
			.audio_in_right(audio_in_right),
			.rst_n(rst_n),
			.clk(clk), 
			.audio_appsel(audio_appsel),
			.audio_sysclk(audio_sysclk),
			.audio_bck(audio_bck),
			.audio_ws(audio_ws),
			.audio_data(audio_data)
		 );
		 
	// Scan control
	scan_ctl U_6(
	  .in0(dis3), // 1st input
	  .in1(dis2), // 2nd input
	  .in2(dis1), // 3rd input
	  .in3(dis0),  // 4th input
	  .ftsd_ctl_en(ftsd_ctl_en), // divided clock for scan control
	  .ftsd_ctl(ftsd_ctl), // ftsd display control signal
	  .ftsd_in(ftsd_in) // output to ftsd display
	);

	ftsd U_7(
	  .in(ftsd_in),  // binary input
	  .display(display) // 14-segment display output
	);


endmodule
